/*************************************************************************/
/* File:        bayer2rgb.v
 * Author:      Michael Phan-Ba
 *
 * Description: Translate bayer pattern to RGB
 *    Bayer pattern:
 *    R G R G
 *    G B G B
 *    R G R G
 *    G B G B
 * Revsions:
 *   1.0        Initial design - 11/30/2009 - Michael Phan-Ba
 **************************************************************************/
`timescale 1ns/1ns

// Convert Bayer pattern to RGB
module bayer2rgb(
  input           clk,
  input           reset,
  input    [10:0] colAddrIn,    // column count
  input    [10:0] lineAddrIn,    // row count
  input     [7:0] pixelIn,
  input           pixelValidIn,   // pixelValidIn
  output    [7:0] red,
  output    [7:0] green,
  output    [7:0] blue,
  output    [8:0] colAddrOut,
  output    [7:0] lineAddrOut,
  output          pixelValidOut);   // pixelValidOut (on 1,1 pixel - divide by 4)

  localparam NUMCOLS = 640;

  // Memory
   reg [7:0] prevBlue [0:NUMCOLS/2 - 1];
   reg [7:0] prevGreen [0:NUMCOLS/2 - 1];
   reg [7:0] prevPixel;

  // Assign output values
  assign red = prevPixel;
  assign green = (prevGreen[colAddrIn>>1] + pixelIn + 1) / 2;
  assign blue = prevBlue[colAddrIn>>1];
  assign colAddrOut = colAddrIn[9:1];
  assign lineAddrOut = lineAddrIn[8:1];
  assign pixelValidOut = pixelValidIn && colAddrIn[0] && lineAddrIn[0];

  // Capture input data
  always @(posedge clk) begin
    if (pixelValidIn) begin
      if (!lineAddrIn[0]) begin
		 if (!colAddrIn[0]) prevGreen[colAddrIn>>1] <= pixelIn;
		 else prevBlue[colAddrIn>>1] <= pixelIn;
	  end
      prevPixel <= pixelIn;
    end
  end

endmodule
