// --------------------------------------------------------------------
// Copyright (c) 2007 by Terasic Technologies Inc.
// --------------------------------------------------------------------
//
// Permission:
//
//   Terasic grants permission to use and modify this code for use
//   in synthesis for all Terasic Development Boards and Altera Development
//   Kits made by Terasic.  Other use of this code, including the selling
//   ,duplication, or modification of any portion is strictly prohibited.

/*****************************************************************************
 * This file is the top-level file for the Altera DE1 project board
 * It implements a camera pipeline using the camera and LCD that come with
 * the board.  This pipeline includes the following modules:
 *
 *    camera - Interface to the external camera
 *    bayer2rgb - Converts the Bayer color pattern to RGB and reduced the image
 *             size by 2 in each dimension
 *    inputAsyncInf - Asynchronous interface between the camera and 
 *             and the frame buffer clock domains
 *    frameBuffer - Frame buffer with 3 frames.  The camera and lcd display
 *             rotate through the 3 frames, following each other.
 *    outputAsyncInf - Asynchonous interface between the frame buffer and the
 *             LCD clock domains
 *    display - Interface to the external LCD display
 * **************************************************************************/

`timescale 1ns/1ns
module de1_d5m
  (////////////////////////    Clock Input     ////////////////////////
   input   [1:0]   CLOCK_24,               //  24 MHz
   input   [1:0]   CLOCK_27,               //  27 MHz
   input           CLOCK_50,               //  50 MHz
   input           EXT_CLOCK,              //  External Clock
   ////////////////////////    Push Button     ////////////////////////
   input   [3:0]   KEY,                    //  Pushbutton[3:0]
   ////////////////////////    DPDT Switch     ////////////////////////
   input   [9:0]   SW,                     //  Toggle Switch[9:0]
   ////////////////////////    7-SEG Dispaly   ////////////////////////
   output  [6:0]   HEX0,                   //  Seven Segment Digit 0
   output  [6:0]   HEX1,                   //  Seven Segment Digit 1
   output  [6:0]   HEX2,                   //  Seven Segment Digit 2
   output  [6:0]   HEX3,                   //  Seven Segment Digit 3
   ////////////////////////////    LED     ////////////////////////////
   output  [7:0]   LEDG,                   //  LED Green[7:0]
   output  [9:0]   LEDR,                   //  LED Red[9:0]
   ////////////////////////////    UART    ////////////////////////////
   output          UART_TXD,               //  UART Transmitter
   input           UART_RXD,               //  UART Receiver
   ///////////////////////     SDRAM Interface ////////////////////////
   inout   [15:0]  DRAM_DQ,                //  SDRAM Data bus 16 Bits
   output  [11:0]  DRAM_ADDR,              //  SDRAM Address bus 12 Bits
   output          DRAM_LDQM,              //  SDRAM Low-byte Data Mask
   output          DRAM_UDQM,              //  SDRAM High-byte Data Mask
   output          DRAM_WE_N,              //  SDRAM Write Enable
   output          DRAM_CAS_N,             //  SDRAM Column Address Strobe
   output          DRAM_RAS_N,             //  SDRAM Row Address Strobe
   output          DRAM_CS_N,              //  SDRAM Chip Select
   output          DRAM_BA_0,              //  SDRAM Bank Address 0
   output          DRAM_BA_1,              //  SDRAM Bank Address 0
   output          DRAM_CLK,               //  SDRAM Clock
   output          DRAM_CKE,               //  SDRAM Clock Enable
   ////////////////////////    Flash Interface ////////////////////////
   inout   [7:0]   FL_DQ,                  //  FLASH Data bus 8 Bits
   output  [21:0]  FL_ADDR,                //  FLASH Address bus 22 Bits
   output          FL_WE_N,                //  FLASH Write Enable
   output          FL_RST_N,               //  FLASH Reset
   output          FL_OE_N,                //  FLASH Output Enable
   output          FL_CE_N,                //  FLASH Chip Enable
   ////////////////////////    SRAM Interface  ////////////////////////
   inout   [15:0]  SRAM_DQ,                //  SRAM Data bus 16 Bits
   output  [17:0]  SRAM_ADDR,              //  SRAM Address bus 18 Bits
   output          SRAM_UB_N,              //  SRAM High-byte Data Mask
   output          SRAM_LB_N,              //  SRAM Low-byte Data Mask
   output          SRAM_WE_N,              //  SRAM Write Enable
   output          SRAM_CE_N,              //  SRAM Chip Enable
   output          SRAM_OE_N,              //  SRAM Output Enable
   ////////////////////    SD Card Interface   ////////////////////////
   inout           SD_DAT,                 //  SD Card Data
   inout           SD_DAT3,                //  SD Card Data 3
   inout           SD_CMD,                 //  SD Card Command Signal
   output          SD_CLK,                 //  SD Card Clock
   ////////////////////////    I2C     ////////////////////////////////
   inout           I2C_SDAT,               //  I2C Data
   output          I2C_SCLK,               //  I2C Clock
   ////////////////////////    PS2     ////////////////////////////////
   input           PS2_DAT,                //  PS2 Data
   input           PS2_CLK,                //  PS2 Clock
   ////////////////////    USB JTAG link   ////////////////////////////
   input           TDI,                    // CPLD -> FPGA (data in)
   input           TCK,                    // CPLD -> FPGA (clk)
   input           TCS,                    // CPLD -> FPGA (CS)
   output          TDO,                    // FPGA -> CPLD (data out)
   ////////////////////////    VGA         ////////////////////////////
   output          VGA_HS,                 //  VGA H_SYNC
   output          VGA_VS,                 //  VGA V_SYNC
   output  [3:0]   VGA_R,                  //  VGA Red[3:0]
   output  [3:0]   VGA_G,                  //  VGA Green[3:0]
   output  [3:0]   VGA_B,                  //  VGA Blue[3:0]
   ////////////////////    Audio CODEC     ////////////////////////////
   inout           AUD_ADCLRCK,            //  Audio CODEC ADC LR Clock
   input           AUD_ADCDAT,             //  Audio CODEC ADC Data
   inout           AUD_DACLRCK,            //  Audio CODEC DAC LR Clock
   output          AUD_DACDAT,             //  Audio CODEC DAC Data
   inout           AUD_BCLK,               //  Audio CODEC Bit-Stream Clock
   output          AUD_XCK,                //  Audio CODEC Chip Clock
   ////////////////////////    GPIO    ////////////////////////////////
   inout   [35:0]  GPIO_0,                 //  GPIO Connection 0
   inout   [35:0]  GPIO_1                 //  GPIO Connection 1
   );

   ////////////////////////    For TFT LCD Module  ///////////////////////
   wire [7:0]      LCM_DATA;       //  LCM Data 8 Bits
   wire            LCM_GRST;       //  LCM Global Reset
   wire            LCM_SHDB;       //  LCM Sleep Mode
   wire            LCM_DCLK;       //  LCM Clcok
   wire            LCM_HSYNC;      //  LCM HSYNC
   wire            LCM_VSYNC;      //  LCM VSYNC
   wire            LCM_SCLK;       //  LCM I2C Clock
   wire            LCM_SDAT;       //  LCM I2C Data
   wire            LCM_SCEN;       //  LCM I2C Enable
   wire            CLK_18;

   assign  GPIO_0[18]  =   LCM_DATA[6];
   assign  GPIO_0[19]  =   LCM_DATA[7];
   assign  GPIO_0[20]  =   LCM_DATA[4];
   assign  GPIO_0[21]  =   LCM_DATA[5];
   assign  GPIO_0[22]  =   LCM_DATA[2];
   assign  GPIO_0[23]  =   LCM_DATA[3];
   assign  GPIO_0[24]  =   LCM_DATA[0];
   assign  GPIO_0[25]  =   LCM_DATA[1];
   assign  GPIO_0[26]  =   LCM_VSYNC;
   assign  GPIO_0[28]  =   LCM_SCLK;
   assign  GPIO_0[29]  =   LCM_DCLK;
   assign  GPIO_0[30]  =   LCM_GRST;
   assign  GPIO_0[31]  =   LCM_SHDB;
   assign  GPIO_0[33]  =   LCM_SCEN;
   assign  GPIO_0[34]  =   LCM_SDAT;
   assign  GPIO_0[35]  =   LCM_HSYNC;

   /************************************************************************
   //   Camera connections
	************************************************************************/
   wire [11:0]     CCD_DATA;
   wire            CCD_SDAT;
   wire            CCD_SCLK;
   wire            CCD_FLASH;
   wire            CCD_FVAL;
   wire            CCD_LVAL;
   wire            CCD_PIXCLK;
   wire            CCD_MCLK;                //  CCD Master Clock
   assign   CCD_DATA[0] =   GPIO_1[13];
   assign   CCD_DATA[1] =   GPIO_1[12];
   assign   CCD_DATA[2] =   GPIO_1[11];
   assign   CCD_DATA[3] =   GPIO_1[10];
   assign   CCD_DATA[4] =   GPIO_1[9];
   assign   CCD_DATA[5] =   GPIO_1[8];
   assign   CCD_DATA[6] =   GPIO_1[7];
   assign   CCD_DATA[7] =   GPIO_1[6];
   assign   CCD_DATA[8] =   GPIO_1[5];
   assign   CCD_DATA[9] =   GPIO_1[4];
   assign   CCD_DATA[10]=   GPIO_1[3];
   assign   CCD_DATA[11]=   GPIO_1[1];
   assign   GPIO_1[16]  =   CLOCK_25;   // 25MHz clock
   //assign GPIO_1[16]  =   CCD_MCLK;   // 25MHz clock
   assign   CCD_FVAL    =   GPIO_1[22];
   assign   CCD_LVAL    =   GPIO_1[21];
   assign   CCD_PIXCLK  =   GPIO_1[0];  // Pixel clock back
   assign   GPIO_1[19]  =   1'b1;       // tRIGGER
   assign   GPIO_1[17]  =   DLY_RST_1;

   //  Set all inout ports to tri-state by default
   assign  FL_DQ       =   8'hzz;
   assign  SRAM_DQ     =   16'hzzzz;
   assign  SD_DAT      =   1'bz;
   assign  I2C_SDAT    =   1'bz;
   assign  AUD_ADCLRCK =   1'bz;
   assign  AUD_DACLRCK =   1'bz;
   assign  AUD_BCLK    =   1'bz;

   // Set unused SRAM pins to default value
   assign SRAM_UB_N = 1'b0;
   assign SRAM_LB_N = 1'b0;
   assign SRAM_CE_N = 1'b0;
   assign SRAM_OE_N = 1'b0;

   /*****************************END of Board Interface************************/
   /***************************************************************************/

   /***************************************************************************/
   /**************************** Clock and Reset Logic ************************/

   parameter SIMULATE=0;    // Top level sets to 1 for simulation

   wire      DLY_RST_0, DLY_RST_1, DLY_RST_2;
   Reset_Delay rd
     (.iCLK(CLOCK_50),
      .iRST(KEY[0]),
      .oRST_0(DLY_RST_0),
      .oRST_1(DLY_RST_1),
      .oRST_2(DLY_RST_2)
      );
   wire      reset = (SIMULATE==0) ? ~DLY_RST_0 : ~KEY[0];
   wire      reset_n = ~reset;

   // Use a PLL to generate 50MHz clock derivatives
   wire      CLOCK_100, CLOCK_25, CLOCK_12_5;
   pll50 p50
     (
      // Outputs
      .c0                               (CLOCK_100),
      .c1                               (CLOCK_25),
      .c2                               (CLOCK_12_5),
      // Inputs
      .inclk0                           (CLOCK_50));

   wire      CLOCK_18;
   // Use a PLL to generate the 18MHz clock for the LCM
   LCM_PLL   p0
     (.inclk0(CLOCK_27[0]),
      .c0(CLOCK_18));

   // Reset signals synchronized to the different clock domains
   reg    reset_27a, reset_27;
   always @(posedge CLOCK_27[0]) begin
      reset_27a <= reset;
      reset_27 <= reset_27a;
   end
   reg reset_25a, reset_25;
   always @(posedge CLOCK_25) begin
      reset_25a <= reset;
      reset_25 <= reset_25a;
   end
   reg reset_18a, reset_18;
   always @(posedge CLOCK_18) begin
      reset_18a <= reset;
      reset_18 <= reset_18a;
   end
   /**********************END of Clock and Reset Logic ************************/
   /***************************************************************************/

   /***************************************************************************/
   /********************** Camera Input****************************************/

   wire  [7:0] ccdPixelOut;
   wire [15:0] ccdColAddr, ccdLineAddr;
   wire        ccdPixelValid;
   wire        ccdNewFrame;

   camera icamera
     (.clk                              (CCD_PIXCLK),   // Camera generates its own clock output
      .reset                            (reset),
      .CCD_DATA                         (CCD_DATA[11:4]),
      .CCD_FVAL                         (CCD_FVAL),
      .CCD_LVAL                         (CCD_LVAL),
      .iSTART                           (!KEY[3]),   // Start and stop buttons for image capture
      .iEND                             (!KEY[2]),
      .pixelOut                         (ccdPixelOut[7:0]),
      .colAddr                          (ccdColAddr[15:0]),
      .lineAddr                         (ccdLineAddr[15:0]),
      .newFrame                         (ccdNewFrame),
      .pixelValid                       (ccdPixelValid));

   wire [8:0]  bayColAddr;
   wire [7:0]  bayLineAddr;
   wire        bayPixelValid;
   wire [7:0]  bayRed, bayGreen, bayBlue;

   bayer2rgb b (
      .clk                              (CCD_PIXCLK),
      .reset                            (reset),
      .colAddrIn                        (ccdColAddr[10:0]),
      .lineAddrIn                       (ccdLineAddr[10:0]),
      .pixelIn                          (ccdPixelOut[7:0]),
      .pixelValidIn                     (ccdPixelValid),
      .red                              (bayRed[7:0]),
      .green                            (bayGreen[7:0]),
      .blue                             (bayBlue[7:0]),
      .colAddrOut                       (bayColAddr),
      .lineAddrOut                      (bayLineAddr),
      .pixelValidOut                    (bayPixelValid));

   frame_counter u5 (
     .reset(reset),
     .clk(CCD_PIXCLK),
     .count(ccdNewFrame),
     .SEG0(HEX0),
     .SEG1(HEX1),
     .SEG2(HEX2),
     .SEG3(HEX3)
   );
   /**********************END of Camera Input *********************************/

   wire [8:0]  filterColAddr;
   wire [7:0]  filterLineAddr;
   wire        filterPixelValid;
   wire [7:0] filterRed, filterGreen, filterBlue;
   

   /***************************************************************************/
   //  	
   /***************************************************************************/
   reg enable;
   always @(posedge CCD_PIXCLK) if (ccdNewFrame) enable <= SW[4];

   filter filter_inst 
     (.clk           (CCD_PIXCLK),
      .enable        (enable),
	  .newFrame      (ccdNewFrame),
	  .pixelValidIn  (bayPixelValid),
	  .lineAddrIn     (bayLineAddr),
	  .colAddrIn     (bayColAddr),
      .pixelRedIn    (bayRed),
      .pixelGreenIn  (bayGreen),
      .pixelBlueIn   (bayBlue),
	  .pixelValidOut (filterPixelValid),
	  .lineAddrOut   (filterLineAddr),
	  .colAddrOut    (filterColAddr),
      .pixelRedOut   (filterRed),
      .pixelGreenOut (filterGreen),
      .pixelBlueOut  (filterBlue)
	  );

   /***************************************************************************/
   /********************** Frame Buffer ***************************************/

   inputAsyncInf #(.SIMULATE(SIMULATE)) inputAsync
     (.inClk                (CCD_PIXCLK),
      .fbClk                (CLOCK_25),
      .fbReset              (reset_25),
      .inReset              (reset_27),
      // Input ports
      .pgLineAddrIn         (filterLineAddr),
      .pgColAddrIn          (filterColAddr),
      .pgRGB                ({ filterRed[7:3], filterGreen[7:2], filterBlue[7:3] }),  // Display color output
      .pgValidPixelIn       (filterPixelValid),
      .pgTakingPixelIn      (/*Ignored*/),
      .pgNewFrameIn         (ccdNewFrame),
      .pgFrameReadyIn       (/*Ignored*/),

      // Output ports
      .fbLineAddrIn(fbLineAddrIn),
      .fbColAddrIn(fbColAddrIn),
      .fbRGBIn(fbRGBIn),
      .fbDataValidIn(fbDataValidIn),
      .fbTakingDataIn(fbTakingDataIn),
      .fbFrameReadyIn(fbFrameReadyIn),
      .fbNewFrameInInt(fbNewFrameInInt));

   wire [7:0]  red, green, blue;
   wire [23:0] dpRGB;
   wire       dpTakingPixelOut, dpValidPixelOut, dpNewFrameOut;

   wire [7:0]     fbLineAddrIn;
   wire [8:0]     fbColAddrIn;
   wire [15:0]    fbRGBIn;
   wire           fbDataValidIn, fbTakingDataIn, fbFrameReadyIn, fbNewFrameInInt, fbDataValidOut,fbTakingDataOut, fbNewFrameOutInt;

   frameBuffer #(.SIMULATE(SIMULATE)) fb
     (.clk                              (CLOCK_25),
      .clk2                             (CLOCK_50),
      .reset                            (reset_25),
      // Input port
      .lineAddrIn                       (fbLineAddrIn[7:0]),
      .colAddrIn                        (fbColAddrIn[8:0]),
      .RGBIn                            (fbRGBIn[15:0]),
      .validPixelIn                     (fbDataValidIn),
      .takingPixelIn                    (fbTakingDataIn),
      .frameReadyIn                     (fbFrameReadyIn),
      .newFrameIn                       (fbNewFrameInInt),
      // Output port
      .red                              (red[7:0]),
      .green                            (green[7:0]),
      .blue                             (blue[7:0]),
      .validPixelOut                    (fbDataValidOut),
      .takingPixelOut                   (fbTakingDataOut),
      .frameReadyOut                    (/*Ignored*/),
      .newFrameOut                      (fbNewFrameOutInt),
      // External SRAM port
      .SRAM_ADDR                        (SRAM_ADDR[17:0]),
      .SRAM_WE_N                        (SRAM_WE_N),
     .SRAM_DQ                           (SRAM_DQ[15:0]));

      wire [23:0]    fbRGBOut = {red, green, blue};

      outputAsyncInf #(.SIMULATE(SIMULATE)) outputAsync
     (.fbClk                (CLOCK_25),
      .outClk               (CLOCK_18),
      .fbReset              (reset_25),
      .outReset             (reset_18),
      // Input ports
      .fbRGBOut(fbRGBOut),
      .fbDataValidOut(fbDataValidOut),
      .fbTakingDataOut(fbTakingDataOut),
      .fbNewFrameOutInt(fbNewFrameOutInt),
      // Output ports
      .dpRGB                (dpRGB[23:0]),
      .dpNewFrameOut        (dpNewFrameOut),
      .dpTakingPixelOut     (dpTakingPixelOut));

   /**********************END of Frame Buffer *********************************/
   /***************************************************************************/
   /********************** LCD Display ****************************************/

   display #(.SIMULATE(SIMULATE)) display
     (.CLK_18      (CLOCK_18),
      .reset       (reset_18),
      // Input from pixel pipeline
      .RGB(dpRGB),
      .takingPixel(dpTakingPixelOut),
      .newFrame(dpNewFrameOut),
      .test(SW[7]),     // Optional test input connected to SW[7]
      //  LCM Interface
      .LCM_DATA(LCM_DATA),
      .LCM_VSYNC(LCM_VSYNC),
      .LCM_HSYNC(LCM_HSYNC),
      .LCM_DCLK(LCM_DCLK),
      .LCM_SHDB(LCM_SHDB),
      .LCM_GRST(LCM_GRST));

   /**********************END of LCD Display **********************************/
   /***************************************************************************/
   /*********** I2C Controllers for camera and LCD ****************************/

   generate if (SIMULATE==0)
     I2C_CCD_Config ccd_config  (   //  Host Side
            .iCLK(CLOCK_50),
            .iRST_N(DLY_RST_2),
            .iZOOM_MODE_SW(~SW[8]),
            .iEXPOSURE_ADJ(KEY[1]),
            .iEXPOSURE_DEC_p(SW[0]),
            .iTEST_PATTERN (SW[9]),
            //  I2C Side
            .I2C_SCLK(GPIO_1[24]),
            .I2C_SDAT(GPIO_1[23])
            );

     I2S_LCM_Config   lcm_config
       (  //  Host Side
          .iCLK(CLOCK_50),
          .iRST_N(KEY[0]),
          //  I2C Side
          .I2S_SCLK(LCM_SCLK),
          .I2S_SDAT(LCM_SDAT),
          .I2S_SCEN(LCM_SCEN) );
   endgenerate

endmodule // de1_d5m
