/*************************************************************************/
/* File:        fifo24.v
 * Author:      Carl Ebeling
*
 * Description: 24-bit wide asynchronous fifo
 * Revsions:
 *   1.0        Initial design - 10/29/2009 - Carl Ebeling
 **************************************************************************/
`timescale 1ns/1ns

`timescale 1ns / 1ns

module fifo24
  (input clkIn, 
   input clkOut,
   input reset,
   input [23:0] dataIn,
   input dataValidIn,
   output takingDataIn,
   output [23:0] dataOut,
   output dataValidOut,
   input takingDataOut);

   wire  wrfull, rdempty;
   
   assign takingDataIn = ~wrfull;
   assign dataValidOut = ~rdempty;
   
   fifo24int fifo 
	 (// Input port
	  .aclr						(reset),
	  .wrclk					(clkIn),
	  .data						(dataIn[23:0]),
	  .wrreq					(dataValidIn),
	  .wrfull					(wrfull),
	  // Output port
	  .rdclk					(clkOut),
	  .rdreq					(takingDataOut),
	  .q						(dataOut[23:0]),
	  .rdempty					(rdempty));

endmodule // fifo24
