/*************************************************************************/
/* File:        fifo33.v
 * Author:      Carl Ebeling
*
 * Description: 16-bit wide asynchronous fifo
 * Revsions:
 *   1.0        Initial design - 10/29/2009 - Carl Ebeling
 **************************************************************************/
`timescale 1ns/1ns

`timescale 1ns / 1ns

module fifo33
  (input clkIn, 
   input clkOut,
   input reset,
   input [32:0] dataIn,
   input dataValidIn,
   output takingDataIn,
   output [32:0] dataOut,
   output dataValidOut,
   input takingDataOut);

   wire  wrfull, rdempty;
   
   assign takingDataIn = ~wrfull;
   assign dataValidOut = ~rdempty;
   
   fifo33int fifo 
	 (// Input port
	  .aclr						(reset),
	  .wrclk					(clkIn),
	  .data						(dataIn[32:0]),
	  .wrreq					(dataValidIn),
	  .wrfull					(wrfull),
	  // Output port
	  .rdclk					(clkOut),
	  .rdreq					(takingDataOut),
	  .q						(dataOut[32:0]),
	  .rdempty					(rdempty));

endmodule // fifo33

