/*************************************************************************/
/* File:        sramDP.v
 * Author:      Carl Ebeling
*
 * Description: Dual-ported SRAM implemented using the single port SRAM
 * Revsions:
 *   1.0        Initial design - 10/17/2009 - Carl Ebeling
 **************************************************************************/
`timescale 1ns/1ns

// Simple dual-port ram.  Read always has priority over write

module sramDP
  (input reset,   // Reset (in RW clock domain)
   input clk,     // Read/Write clock
   input clk2,    // Double frequency clock for timing generation
   input write,   // Write request
   output takingWrite,  // If we actually did the write
   input [17:0] wrAddr,
   input [15:0] wrData,
   input read,    // Read request
   input [17:0] rdAddr,
   output [15:0] rdData,
   
   inout	[15:0]	SRAM_DQ,			//	SRAM Data bus 16 Bits
   output	[17:0]	SRAM_ADDR,			//	SRAM Address bus 18 Bits
   output			SRAM_WE_N			//	SRAM Write Enable
);

   reg 				phi1, phi2;
   // Generate the WE pulse
   always @(negedge clk2) begin
	  phi1 <= clk;
   end
   always @(posedge clk2) begin
	  phi2 <= phi1;
   end
   wire WEpulse = phi1 & phi2;
   
   assign SRAM_WE_N = ~(WEpulse & write & ~read);
   assign SRAM_ADDR = read ? rdAddr : wrAddr;
   assign SRAM_DQ = read ? 16'hzzzz : wrData;
   assign takingWrite = ~read;
   
   assign rdData = SRAM_DQ;

endmodule // sramDP
